//---------------------------------------------------
// File name   : snmakedeps.e
// Project     : AEWare
// Developers  : Steve Hobbs, Cadence Design Systems
// Created     : Thu Nov 27 18:53:27 GMT 2008
// Description : Specman Makefile dependency generator
// Notes       : Load this or compile it into Specman,
//             : then use the command:
//             : "write makefile <filename> [-ignore_test]"
// --------------------------------------------------
// Copyright 2008 (c) Cadence Design Systems
// --------------------------------------------------
<'
struct epkg {
  !name : string;
  !filelist : list of string;
};

extend sys {
  snmakedeps(filename : string, ignore_test_files : bool) is {
    var pkgs       : list(key:name) of epkg;
    var compmods   : list of rf_module;
    var top_module : rf_module;
    var makefile   : file = files.open(filename, "w", "Makefile");
    out("Generating Makefile dependencies for loaded a modules.");

    files.write(makefile, "## Auto-generated Makefile for Specman - DO NOT EDIT");

    for each (m) in rf_manager.get_user_modules() {
      var modulename : string = m.get_name();
      if ( (modulename ~ "snmakedeps") ) || ( ignore_test_files && (modulename ~ "/^test/") ) {
        outf("Ignoring file %s\n", modulename);
      } else {
        var filename : string = m.get_full_file_name();
        var pkgname  : string = m.get_package().get_name();
        top_module = m;
        if (! pkgs.key_exists(pkgname)) {
          var p : epkg = new;
          p.name = pkgname;
          p.filelist = {filename};
          pkgs.add(p);
        } else {
          pkgs.key(pkgname).filelist.add(filename);
        };
        if ( m.is_compiled() && m.is_top_module() ) { compmods.add(m); };
      };
    };
    for each (p) in pkgs {
      files.write(makefile, appendf("## Package %s\n%s=\\", p.name, p.name));
      for i from 0 to p.filelist.size()-2 do {
        files.write(makefile, appendf("\t%s \\", p.filelist[i]));
      };
      files.write(makefile, appendf("\t%s\n", p.filelist[p.filelist.size()-1]));  // last item has no trailing backslash!
    };

    var ecode : string = "allecode=";
    for each (p) in pkgs { ecode = appendf("%s$(%s) ", ecode, p.name); };
    files.write(makefile, appendf("%s\n",ecode));

    // Generate sn_compile targets for each compiled chunk
    // TODO: generate dependency on all preceding modules via get_index() method...
    for each (m) in compmods {
      files.write(makefile, appendf("libsn_%s.so: %s", m.get_name(), m.get_full_file_name()));
      files.write(makefile, appendf("\tsn_compile.sh -exe -shlib -enable_DAC %s\n", m.get_full_file_name()));
    };
    files.write(makefile, appendf("compile: libsn_%s.so\n\techo 'Recompiling e code...'\n\n", compmods[compmods.size()-1].get_name()));

    // Build a Make target for generating any chosen stub file
    // Chooses the run the compiled exe if it exists, else loads the e code
    files.write(makefile, appendf("# Use 'make stub_<sn_adapter_name>' to build the desired stubs file,"));
    files.write(makefile, appendf("# for example: 'make stub_ncvhdl'."));
    files.write(makefile, appendf("stub_%%: $(allecode)"));
    files.write(makefile, appendf("\tif [ -x ./%s ]; then\\\n\t ./%s -c 'load %s ; write stubs -$*'\\",
                                  compmods[compmods.size()-1].get_name(), compmods[compmods.size()-1].get_name(), top_module.get_full_file_name()));
    files.write(makefile, appendf("\telse\\\n\tspecman -c 'load %s ; write stubs -$*';\\\n\tfi\n",
                                  top_module.get_full_file_name()));

    files.close(makefile);
  };
};

define <write_makefile_dependencies'command> "write makefile <filename'file>[ <ignore_test'exp>]" as computed {
  if (<ignore_test'exp> == "-ignore_test") || (<ignore_test'exp> == "-ignore") {
    return appendf("sys.snmakedeps(\"%s\",TRUE)",<filename'file>);
  } else {
    return appendf("sys.snmakedeps(\"%s\",FALSE)",<filename'file>);
  };
};
'>

